local select = select
local build = strjoin(" - ", GetBuildInfo())
local class = select(2, UnitClass("player"))

local function checkdiffs()
	if select(5, GetAddOnInfo("Talented_Data")) and Talented_Data then
		local messages = {}
		local function msg(s, ...)
			if s:find("%", 1, 1) then
				s = s:format(...)
			end
			table.insert(messages, s)
		end
		local function assert(v, ...)
			if not v then
				msg(...)
			end
		end
		if not IsAddOnLoaded("Talented_Data") then
			LoadAddOn("Talented_Data")
		end
		local source, dest = Talented_GatherDB[build][class], Talented_Data[class]
		assert(#source == #dest, "Different number of Talent Tabs for class %s", class)
		for tab = 1, math.min(#source, #dest) do
			local sinfo, dinfo = source[tab].info, dest[tab].info
			assert(sinfo.icon == dinfo.icon, "Tab %s:%d has a new icon", class, tab)
			assert(sinfo.background == dinfo.background, "Tab %s:%d has a new background", class, tab)
			local name = GetTalentTabInfo(tab)
			assert(name == dinfo.name, "Tab %s:%d has a new name", class, tab)
			
			local stalents, dtalents = source[tab].talents, dest[tab].talents
			assert(#stalents == #dtalents, "Different number of talents for tab %s:%d", class, tab)
			for index = 1, math.min(#stalents, #dtalents) do
				local sinfo, dinfo = stalents[index].info, dtalents[index].info
				assert(sinfo.icon == dinfo.icon, "Talent %s:%d:%d has a new icon", class, tab, index)
				assert(sinfo.row == dinfo.row, "Talent %s:%d:%d has a new row", class, tab, index)
				assert(sinfo.column == dinfo.column, "Talent %s:%d:%d has a new column", class, tab, index)
				assert(sinfo.ranks == dinfo.ranks, "Talent %s:%d:%d has a new ranks", class, tab, index)
				assert(sinfo.exceptional == dinfo.exceptional, "Talent %s:%d:%d has a new exceptional", class, tab, index)
				local name = GetTalentInfo(tab, index)
				assert(name == dinfo.name, "Talent %s:%d:%d has a new name", class, tab, index)
				assert(type(sinfo.prereqs) == type(dinfo.prereqs), "Talent %s:%d:%d has a different type of prereqs", class, tab, index)
				if type(sinfo.prereqs) == "table" and type(dinfo.prereqs) == "table" then
					assert(#sinfo.prereqs == #dinfo.prereqs, "Talent %s:%d:%d has a different number of prereqs", class, tab, index)
					for p = 1, math.min(#sinfo.prereqs, #dinfo.prereqs) do
						assert(sinfo.prereqs[p].row == dinfo.prereqs[p].row, "Prereq %s:%d:%d:%d has a new row", class, tab, index, p)
						assert(sinfo.prereqs[p].column == dinfo.prereqs[p].column, "Prereq %s:%d:%d:%d has a new row", class, tab, index, p)
					end
				end
			end
		end
		if #messages > 0 then
			ChatFrame1:AddMessage("Talented_GatherData has detected differences !")
			ChatFrame1:AddMessage("  ----------  ")
			for _, s in ipairs(messages) do
				ChatFrame1:AddMessage(s)
			end
			ChatFrame1:AddMessage("  ----------  ")
			Talented_GatherDB[build][class].differences = messages
		end
	end
end

local FillPrereqs = function (...)
	if select("#", ...) < 3 then return end
	local table = {}
	for i = 1, select("#", ...), 3 do
		local row, column = select(i, ...)
		table[#table + 1] = { row = row, column = column }
	end
	return table
end

local function GetClassTalentsInfo()
	local result = {} 
	local count = GetNumTalentTabs()
	if count == 0 then return end
	for tab = 1, GetNumTalentTabs() do
		local icon, _, background = select(2, GetTalentTabInfo(tab))
		result[tab] = {
			info = {
				icon = icon,
				background = background,
			},
			talents = {}
		}
		local talents = result[tab].talents
		for index = 1, GetNumTalents(tab) do
			local icon, row, column, _, ranks, exceptional = select(2, GetTalentInfo(tab, index))
			talents[index] = {
				info = {
					icon = icon,
					row = row,
					column = column,
					ranks = ranks,
					exceptional = exceptional,
					prereqs = FillPrereqs(GetTalentPrereqs(tab, index)),
				},
			}
		end
	end
	Talented_GatherDB[build][select(2, UnitClass("player"))] = result
	checkdiffs()
	return true
end

local f = CreateFrame("Frame", nil, UIParent)

function f:OnUpdate(elapsed)
	self.time = self.time - elapsed
	if self.time < 0 then
		self:UpdateTalents()
	end
end

function f:UpdateTalents()
	self:SetScript("OnUpdate", nil)
	if GetClassTalentsInfo() then
		self:SetScript("OnEvent", nil)
		self:Hide()
	else
		self.time = 1
		self:SetScript("OnUpdate", self.OnUpdate)
	end
end

function f:PLAYER_LOGIN()
	self:UpdateTalents()
end

function f:ADDON_LOADED(addon)
	if addon == "Talented_GatherData" then
		Talented_GatherDB = Talented_GatherDB or {}
        Talented_GatherDB.latest = build
		Talented_GatherDB[build] = Talented_GatherDB[build] or {}
		Talented_GatherDB[build][class] = nil
	end
end

function f:OnEvent(event, ...)
	local handler = self[event]
	if type(handler) == "function" then
		handler(self, ...)
	end
end

f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("ADDON_LOADED")
f:SetScript("OnEvent", f.OnEvent)
