
Compressor = {}
local Compressor = Compressor

function Compressor:init()
	self.dict = {}
end

local submit_tokens = function (dict, ...)
	for i = 1, select("#", ...) do
		local s = select(i, ...)
		local v = (dict[s] or 0)
		dict[s] = v + 1
	end
end

function Compressor:submit(s)
	submit_tokens(self.dict, (" "):split(s))
end

function Compressor:sort_tokens()
	local dict = self.dict
	local tmp = {}
	for token, count in pairs(dict) do
		tmp[#tmp + 1] = {token, count}
	end
	local unpack = unpack
	table.sort(tmp, 
		function (a, b)
			local tok1, count1 = unpack(a)
			local tok2, count2 = unpack(b)
			if count1 ~= count2 then
				return count1 > count2
			elseif tok1:len() ~= tok2:len() then
				return tok1:len() < tok2:len()
			else
				return tok1 < tok2
			end
		end)
	
	for index, entry in ipairs(tmp) do
		dict[entry[1]] = index
	end
	self.sorteddict = tmp
end

local floor = math.floor
local fmod = math.fmod
local char = string.char

local compress_tokens = function (dict, ...)
	r = {}
	for i = 1, select("#", ...) do
		local index = dict[select(i, ...)]
		if index < 128 then
			r[#r + 1] = char(index)
		elseif index < 16384 then -- 2^(6+8)
			local a = floor(index / 256)
			local b = fmod(index, 256)
			r[#r + 1] = char(128 + a, b)
		elseif index < 2097152 then -- 2^(5+8+8)
			local a = floor(index / 65536)
			local t = fmod(index, 65536)
			local b = floor(t, 256)
			local c = fmod(t, 256)
			r[#r + 1] = char(192 + a, b, c)
		else
			error("token dict too big")
		end
	end
	return table.concat(r)
end

function Compressor:compress(s)
	return compress_tokens(self.dict, (" "):split(s))
end

local decompressor_start = [[
local setmetatable = setmetatable
local type = type
local tostring = tostring
local rawget = rawget
local rawset = rawset
local pairs = pairs
local next = next
local ipairs = ipairs
local concat = table.concat
local len = string.len
local byte = string.byte

local dictionary
local temp_decomptable = {}
local decompress = function (s)
	local i = 1
	local pos = 1
	local len = s:len()
	while pos <= len do
		local a, b, c, index
		a = byte(s, pos)
		if a < 128 then
			index = a
			pos = pos + 1
		elseif a < 192 then
			b = byte(s, pos + 1)
			index = (a - 128) * 256 + b
			pos = pos + 2
		else
			b, c = byte(s, pos + 1, pos + 2)
			index = (a - 192) * 65536 + b * 256 + c
			pos = pos + 3
		end
		temp_decomptable[i] = dictionary[index]
		i = i + 1
	end
	while temp_decomptable[i] do
		temp_decomptable[i] = nil
		i = i + 1
	end
	return concat(temp_decomptable, " ")
end

decompMT = {
	__index = function(table, key)
		local ckey = tostring(key)
		if not ckey then return end
		ckey = "compressed-"..ckey
		local comp = rawget(table, ckey)
		if not comp then return end
		local value = decompress(comp)
		rawset(table, key, value)
		rawset(table, ckey, nil)
		return value
	end
}

dictionary = setmetatable({
]]

local decompressor_entry = [[
	%q, -- %d, %d
]]

local decompressor_end = [[
}, { __mode = "kv" })

for _, trees in pairs(Talented_Data) do
	for _, tree in ipairs(trees) do
		setmetatable(tree.info, decompMT)
		for _, talent in ipairs(tree.talents) do
			setmetatable(talent.info, decompMT)
			local first_tip = next(talent.info.tips)
			if type(first_tip) == "string" then
				setmetatable(talent.info.tips, decompMT)
			else
				for _, lines in ipairs(talent.info.tips) do
					for _, line in ipairs(lines) do
						setmetatable(line, decompMT)
					end
				end
			end
		end
	end
end
]]

function Compressor:write_decompressor(f)
	f:write(decompressor_start)
	for i, v in ipairs(self.sorteddict) do
		f:write(decompressor_entry:format(v[1], i, v[2]))
	end
	f:write(decompressor_end)
end
