--[[
	HOWTO : 
	You must install the addon "Talented_GatherData" in wow, then log
	a character from each of the 9 classes.
	After this, you must copy the file :
	<WOW_PATH>\WTF\<ACCOUNT>\SavedVariables\Talented_GatherData.lua
	in this directory, then run lua convert-sv.lua to generate the final
	class database files.
]]
dofile("Talented_GatherData.lua")

local build = Talented_GatherDB.latest

local classes = {
	DRUID = true,
	HUNTER = true,
	MAGE = true,
	PALADIN = true,
	PRIEST = true,
	ROGUE = true,
	SHAMAN = true,
	WARRIOR = true,
	WARLOCK = true,
}

local function write_talent(o, talent)
	o:write(string.format([[
			{
				info = {
					icon = %q,
					row = %d,
					column = %d,
					ranks = %d,
]], talent.info.icon, talent.info.row, talent.info.column, talent.info.ranks))
	if talent.info.exceptional then
		o:write("\t\t\t\t\texceptional = 1,\n")
	end
	
	local prereqs = talent.info.prereqs
	if prereqs and next(prereqs) then
		o:write("\t\t\t\t\tprereqs = {\n")
		for _, r in ipairs(prereqs) do
			o:write(string.format([[
						{
							row = %d,
							column = %d,
						},
]], r.row, r.column))
		end
		o:write("\t\t\t\t\t},\n")
	end

	o:write([[
				},
			},
]])

end

local function write_tree(o, tree)
	o:write(string.format([[
	{
		info = {
			icon = %q,
			background = %q,
		},
		talents = {
]], tree.info.icon, tree.info.background))
	for _, talent in ipairs(tree.talents) do
		write_talent(o, talent)
	end
	o:write([[
		},
	},
]])
end

for c in pairs(classes) do
	local info = Talented_GatherDB[build][c]
	if info then
		o = io.open(string.format("../Talented_Data_%s.lua", c), "w")
		o:write(string.format([[
-- Generated Data, do not modify 
if not Talented_Data then return end

Talented_Data.%s = {
]], c))
		for _, tab in ipairs(info) do
			write_tree(o, tab)
		end
		o:write("}\n")
		o:close()
	end
end

f = io.open("../Talented_Data_init.lua", "w")
f:write(string.format([[
if strjoin(" - ", GetBuildInfo()) ~= %q then
	DisableAddOn("Talented_Data")
	DEFAULT_CHAT_FRAME:AddMessage("Wrong version, Talented_Data has been desactivated")
else
	Talented_Data = {}
end
]], build))
f:close()

f = io.open("../Talented_Data.toc", "w")
f:write(string.format([[
## Interface: 20100
## Title: Talented_Data
## Name: Talented_Data
## Notes: Class Data for The Talent Template Editor
## Author: Jerry
## Version: 1.0
## X-AceForum: 3955
## X-Category: Interface Enhancements
## X-ExpectedBuild: %q
## LoadOnDemand: 1

Talented_Data_init.lua

Talented_Data_HUNTER.lua
Talented_Data_WARRIOR.lua
Talented_Data_ROGUE.lua
Talented_Data_MAGE.lua
Talented_Data_PRIEST.lua
Talented_Data_WARLOCK.lua
Talented_Data_DRUID.lua
Talented_Data_PALADIN.lua
Talented_Data_SHAMAN.lua

Talented_Data_enUS.lua
Talented_Data_enGB.lua
Talented_Data_deDE.lua
Talented_Data_frFR.lua
Talented_Data_esES.lua
Talented_Data_koKR.lua
]], build))
f:close()